@@ -23,7 +23,7 @@
  * Free Software Foundation, Inc., 59 Temple Place, Suite 330,
  * Boston, MA  02111-1307  USA
  *
- * $Id: gtkimcontextscim.cpp,v 1.170.2.8 2006/09/29 15:17:48 suzhe Exp $
+ * $Id: gtkimcontextscim.cpp,v 1.170.2.13 2007/06/16 06:23:38 suzhe Exp $
  */
 
 #define Uses_SCIM_DEBUG
@@ -36,10 +36,11 @@
 #define Uses_SCIM_TRANSACTION
 #define Uses_SCIM_HOTKEY
 #define Uses_SCIM_PANEL_CLIENT
-#include <iostream>
+#define Uses_C_STRING
+#define Uses_C_STDIO
+#define Uses_C_STDLIB
+#define Uses_STL_IOSTREAM
 
-#include <stdio.h>
-#include <stdlib.h>
 #include <sys/types.h>
 #include <sys/wait.h>
 #include <sys/time.h>
@@ -94,6 +95,7 @@ struct _GtkIMContextSCIMImpl
     bool                     is_on;
     bool                     shared_si;
     bool                     preedit_started;
+    bool                     preedit_updating;
 
     GtkIMContextSCIMImpl    *next;
 };
@@ -541,6 +543,7 @@ gtk_im_context_scim_init (GtkIMContextSCIM      *context_scim,
     context_scim->impl->shared_si = _shared_input_method;
     context_scim->impl->use_preedit = _on_the_spot;
     context_scim->impl->preedit_started = false;
+    context_scim->impl->preedit_updating = false;
 
     context_scim->id = _context_count++;
 
@@ -670,15 +673,22 @@ gtk_im_context_scim_reset (GtkIMContext *context)
 static void
 gtk_im_context_scim_focus_in (GtkIMContext *context)
 {
-    SCIM_DEBUG_FRONTEND(1) << "gtk_im_context_scim_focus_in...\n";
-
     GtkIMContextSCIM *context_scim = GTK_IM_CONTEXT_SCIM (context);
 
-    if (_focused_ic)
+    SCIM_DEBUG_FRONTEND(1) << "gtk_im_context_scim_focus_in(" << context_scim->id << ")...\n";
+
+    if (_focused_ic) {
+        if (_focused_ic == context_scim) {
+            SCIM_DEBUG_FRONTEND(1) << "It's already focused.\n";
+            return;
+        }
+        SCIM_DEBUG_FRONTEND(1) << "Focus out previous IC first: " << _focused_ic->id << "\n";
         gtk_im_context_scim_focus_out (GTK_IM_CONTEXT (_focused_ic));
+    }
 
     // Only use key snooper when use_key_snooper option is enabled and a gtk main loop is running.
     if (_use_key_snooper && !_snooper_installed && gtk_main_level () > 0) {
+        SCIM_DEBUG_FRONTEND(2) << "Install key snooper.\n";
         _snooper_id = gtk_key_snooper_install ((GtkKeySnoopFunc)gtk_scim_key_snooper, NULL);
         _snooper_installed = true;
     }
@@ -759,15 +769,16 @@ gtk_im_context_scim_focus_in (GtkIMContext *context)
 static void
 gtk_im_context_scim_focus_out (GtkIMContext *context)
 {
-    SCIM_DEBUG_FRONTEND(1) << "gtk_im_context_scim_focus_out...\n";
+    GtkIMContextSCIM *context_scim = GTK_IM_CONTEXT_SCIM (context);
+
+    SCIM_DEBUG_FRONTEND(1) << "gtk_im_context_scim_focus_out(" << context_scim->id << ")...\n";
 
     if (_snooper_installed) {
+        SCIM_DEBUG_FRONTEND(2) << "Remove key snooper.\n";
         gtk_key_snooper_remove (_snooper_id);
         _snooper_installed = false;
     }
 
-    GtkIMContextSCIM *context_scim = GTK_IM_CONTEXT_SCIM (context);
-
     if (context_scim && context_scim->impl && context_scim == _focused_ic) {
         _panel_client.prepare (context_scim->id);
         context_scim->impl->si->focus_out ();
@@ -789,6 +800,10 @@ gtk_im_context_scim_set_cursor_location (GtkIMContext *context,
     GtkIMContextSCIM *context_scim = GTK_IM_CONTEXT_SCIM (context);
  
     if (context_scim && context_scim->impl && context_scim->impl->client_window && context_scim == _focused_ic) {
+        // Don't update spot location while updating preedit string.
+        if (context_scim->impl->preedit_updating)
+            return;
+
         gdk_window_get_origin(context_scim->impl->client_window, &x, &y);
         if (context_scim->impl->cursor_x != x + area->x + area->width ||
             context_scim->impl->cursor_y != y + area->y + area->height + 8) {
@@ -976,6 +991,9 @@ gtk_scim_key_snooper (GtkWidget    *grab_widget,
 
         if (!filter_hotkeys (_focused_ic, key)) {
             if (!_focused_ic->impl->is_on || !_focused_ic->impl->si->process_key_event (key)) {
+                SCIM_DEBUG_FRONTEND(3) << "Failed to process: "
+                                       << (!_focused_ic->impl->is_on ? "IC is off" : "IC doesn't process")
+                                       << ".\n";
                 ret = _fallback_instance->process_key_event (key);
             } else {
                 ret = TRUE;
@@ -987,6 +1005,11 @@ gtk_scim_key_snooper (GtkWidget    *grab_widget,
         _panel_client.send ();
 
         _focused_widget = 0;
+    } else {
+        SCIM_DEBUG_FRONTEND(3) << "Failed snooper: "
+                               << ((!_focused_ic || !_focused_ic->impl) ? "Invalid focused ic" :
+                                   (event->send_event ? "send event is set" : "unknown"))
+                               << "\n";
     }
 
     return ret;
@@ -1113,7 +1136,8 @@ panel_slot_process_key_event (int context, const KeyEvent &key)
         _panel_client.prepare (ic->id);
 
         if (!filter_hotkeys (ic, key)) {
-            if (!_focused_ic->impl->is_on || !_focused_ic->impl->si->process_key_event (key)) {
+            if (!_focused_ic || !_focused_ic->impl->is_on ||
+                !_focused_ic->impl->si->process_key_event (key)) {
                 if (!_fallback_instance->process_key_event (key)) {
                     // Just send it to key_snooper and bypass to client directly (because send_event is set to TRUE).
                     GdkEventKey gdkevent = keyevent_scim_to_gdk (ic, key);
@@ -1464,6 +1488,18 @@ keyevent_gdk_to_scim (const GtkIMContextSCIM *ic, const GdkEventKey &gdkevent)
         display = GDK_DISPLAY_XDISPLAY (gdk_display_get_default ());
 
     key.mask = scim_x11_keymask_x11_to_scim (display, gdkevent.state);
+
+    // Special treatment for two backslash keys on jp106 keyboard.
+    if (key.code == SCIM_KEY_backslash) {
+        int keysym_size = 0;
+        KeySym *keysyms = XGetKeyboardMapping (display, gdkevent.hardware_keycode, 1, &keysym_size);
+        if (keysyms != NULL) {
+            if (keysyms[0] == XK_backslash &&
+                (keysym_size > 1 && keysyms[1] == XK_underscore))
+                key.mask |= SCIM_KEY_QuirkKanaRoMask;
+            XFree (keysyms);
+        }
+    }
 #else
     if (gdkevent.state & GDK_SHIFT_MASK) key.mask |=SCIM_KEY_ShiftMask;
     if (gdkevent.state & GDK_LOCK_MASK) key.mask |=SCIM_KEY_CapsLockMask;
@@ -2084,7 +2120,10 @@ slot_update_preedit_string (IMEngineInstanceBase *si,
                 g_signal_emit_by_name(_focused_ic, "preedit-start");
                 ic->impl->preedit_started = true;
             }
+            ic->impl->preedit_caret    = str.length ();
+            ic->impl->preedit_updating = true;
             g_signal_emit_by_name(ic, "preedit-changed");
+            ic->impl->preedit_updating = false;
         } else {
             _panel_client.update_preedit_string (ic->id, str, attrs);
         }
@@ -2291,6 +2330,8 @@ reload_config_callback (const ConfigPointer &config)
 
     _valid_key_mask = (key.mask > 0)?(key.mask):0xFFFF;
     _valid_key_mask |= SCIM_KEY_ReleaseMask;
+    // Special treatment for two backslash keys on jp106 keyboard.
+    _valid_key_mask |= SCIM_KEY_QuirkKanaRoMask;
 
     _on_the_spot = config->read (String (SCIM_CONFIG_FRONTEND_ON_THE_SPOT), _on_the_spot);
     _shared_input_method = config->read (String (SCIM_CONFIG_FRONTEND_SHARED_INPUT_METHOD), _shared_input_method);
