@@ -19,7 +19,7 @@
  * Free Software Foundation, Inc., 59 Temple Place, Suite 330,
  * Boston, MA  02111-1307  USA
  *
- * $Id: scim_setup_ui.cpp,v 1.52 2005/06/23 05:12:58 suzhe Exp $
+ * $Id: scim_setup_ui.cpp,v 1.52.2.1 2007/05/04 02:17:36 juhp Exp $
  *
  */
 
@@ -453,7 +453,7 @@ SetupUI::ask_quit () const
                                      GTK_DIALOG_MODAL,
                                      GTK_MESSAGE_WARNING,
                                      GTK_BUTTONS_OK_CANCEL,
-                                     _("Are you sure to quit SCIM Setup?"));
+                                     _("Are you sure you want to quit SCIM Setup?"));
 
     result = gtk_dialog_run (GTK_DIALOG (dialog));
 
