@@ -4,7 +4,7 @@
 /* 
  * Smart Common Input Method
  * 
- * Copyright (c) 2002 James Su <suzhe@turbolinux.com.cn>
+ * Copyright (c) 2002-2005 James Su <suzhe@tsinghua.org.cn>
  *
  *
  * This library is free software; you can redistribute it and/or
@@ -22,7 +22,7 @@
  * Free Software Foundation, Inc., 59 Temple Place, Suite 330,
  * Boston, MA  02111-1307  USA
  *
- * $Id: scim_setup_ui.h,v 1.17 2004/06/09 15:13:08 suzhe Exp $
+ * $Id: scim_setup_ui.h,v 1.23 2005/01/10 08:30:45 suzhe Exp $
  */
 
 #if !defined (__SCIM_SETUP_UI_H)
@@ -44,52 +44,34 @@ class SetupUI
     GtkTreeSelection *m_module_list_selection;
     GtkTreeStore     *m_module_list_model;
 
-    GtkWidget        *m_factory_list_view;
-    GtkTreeStore     *m_factory_list_model;
-
     GtkWidget        *m_current_widget;
     SetupModule      *m_current_module;
 
     ConfigPointer     m_config;
-    String            m_config_name;
-
-    std::vector <std::pair <String, ConfigPointer> > m_all_configs;
 
     guint             m_query_changed_timeout;
 
     bool              m_changes_applied;
-    bool              m_factory_list_changed;
+
+    HelperAgent       m_helper_agent;
 
     SetupUI (const SetupUI &);
     SetupUI & operator= (const SetupUI &);
 
 public:
-    SetupUI (int argc, char *argv []);
+    SetupUI (const ConfigPointer &config, const String &display, const HelperInfo &helper_info);
     ~SetupUI ();
 
     bool add_module (SetupModule *module);
 
     // Return true if the changes have been applied.
-    bool run ();
-
+    void run ();
 private:
-    void load_config_modules (const String &def_config);
-
     void create_main_ui ();
     void create_module_list_model ();
-    void create_factory_list_model ();
-    void get_factory_list (std::vector <String> &uuids,
-                           std::vector <String> &names,
-                           std::vector <String> &langs,
-                           std::vector <String> &icons);
 
     GtkWidget * create_splash_view ();
     GtkWidget * create_setup_cover (const char *category);
-    GtkWidget * create_factory_list_view ();
-
-    void load_factory_list_config ();
-    void save_factory_list_config ();
-    void factory_list_update_inconsistent (void);
 
     bool ask_quit () const;
 
@@ -112,12 +94,6 @@ private:
     static gboolean module_list_save_config_iter_func (GtkTreeModel *model, GtkTreePath *path, GtkTreeIter *iter, gpointer data); 
 
     static gboolean module_list_load_config_iter_func (GtkTreeModel *model, GtkTreePath *path, GtkTreeIter *iter, gpointer data); 
-
-    static void factory_list_changed_callback (GtkCellRendererToggle *cell, gchar *arg1, gpointer data);
-
-    static gboolean factory_list_load_config_func (GtkTreeModel *model, GtkTreePath *path, GtkTreeIter *iter, gpointer data);
-
-    static gboolean factory_list_save_config_func (GtkTreeModel *model, GtkTreePath *path, GtkTreeIter *iter, gpointer data);
 };
 
 #endif // __SCIM_SETUP_UI_H
