@@ -5,7 +5,7 @@
 /*
  * Smart Common Input Method
  * 
- * Copyright (c) 2002 James Su <suzhe@turbolinux.com.cn>
+ * Copyright (c) 2002-2005 James Su <suzhe@tsinghua.org.cn>
  *
  *
  * This library is free software; you can redistribute it and/or
@@ -23,32 +23,43 @@
  * Free Software Foundation, Inc., 59 Temple Place, Suite 330,
  * Boston, MA  02111-1307  USA
  *
- * $Id: scim_simple_config.h,v 1.16 2004/05/13 09:03:45 suzhe Exp $
+ * $Id: scim_simple_config.h,v 1.22 2005/07/06 03:57:04 suzhe Exp $
  */
 
 #if !defined (__SCIM_SIMPLE_CONFIG_H)
 #define __SCIM_SIMPLE_CONFIG_H
 
 #include <sys/time.h>
+#include "scim_stl_map.h"
 
-using namespace scim;
+namespace scim {
 
 const int SCIM_MAX_CONFIG_LINE_LENGTH = 16384;
 
 class SimpleConfig : public ConfigBase
 {
-    std::map<String, String> m_config;
-    std::map<String, String> m_new_config;
+#if SCIM_USE_STL_EXT_HASH_MAP
+typedef __gnu_cxx::hash_map <String, String, scim_hash_string> KeyValueRepository;
+#elif SCIM_USE_STL_HASH_MAP
+typedef std::hash_map <String, String, scim_hash_string> KeyValueRepository;
+#else
+typedef std::map <String, String> KeyValueRepository;
+#endif
+
+    KeyValueRepository       m_config;
+    KeyValueRepository       m_new_config;
     std::vector <String>     m_erased_keys;
     timeval                  m_update_timestamp;
     bool                     m_need_reload;
 
 public:
-    SimpleConfig (const String& app_name = String ("scim"));
+    SimpleConfig ();
 
     virtual ~SimpleConfig ();
 
     virtual bool valid () const;
+
+    virtual String get_name () const;
     
     // String
     virtual bool read (const String& key, String *pStr) const;
@@ -94,12 +105,17 @@ private:
     String get_param_portion (const String &str);
     String get_value_portion (const String &str);
 
-    void parse_config (std::istream &is, std::map <String, String> &config);
+    void parse_config (std::istream &is, KeyValueRepository &config);
 
     void save_config (std::ostream &os);
 
     bool load_all_config ();
+
+    void remove_key_from_erased_list (const String &key);
 };
+
+} // namespace scim
+
 #endif
 
 /*
