@@ -23,7 +23,7 @@
  * Free Software Foundation, Inc., 59 Temple Place, Suite 330,
  * Boston, MA  02111-1307  USA
  *
- * $Id: scim_panel_gtk_setup.cpp,v 1.14 2005/01/10 08:30:48 suzhe Exp $
+ * $Id: scim_panel_gtk_setup.cpp,v 1.15 2005/12/10 14:04:54 suzhe Exp $
  *
  */
 
@@ -49,6 +49,7 @@ using namespace scim;
 
 #define SCIM_CONFIG_PANEL_GTK_FONT                      "/Panel/Gtk/Font"
 #define SCIM_CONFIG_PANEL_GTK_TOOLBAR_ALWAYS_SHOW       "/Panel/Gtk/ToolBar/AlwaysShow"
+#define SCIM_CONFIG_PANEL_GTK_TOOLBAR_ALWAYS_HIDDEN     "/Panel/Gtk/ToolBar/AlwaysHidden"
 #define SCIM_CONFIG_PANEL_GTK_TOOLBAR_AUTO_SNAP         "/Panel/Gtk/ToolBar/AutoSnap"
 #define SCIM_CONFIG_PANEL_GTK_TOOLBAR_HIDE_TIMEOUT      "/Panel/Gtk/ToolBar/HideTimeout"
 #define SCIM_CONFIG_PANEL_GTK_TOOLBAR_POS_X             "/Panel/Gtk/ToolBar/POS_X"
@@ -117,6 +118,7 @@ extern "C" {
 
 // Internal data declaration.
 static bool   __config_toolbar_always_show       = false;
+static bool   __config_toolbar_always_hidden     = false;
 static bool   __config_toolbar_auto_snap         = false;
 static int    __config_toolbar_hide_timeout      = 2;
 static bool   __config_toolbar_show_factory_icon = true;
@@ -134,7 +136,7 @@ static String __config_font                      = "default";
 
 static bool   __have_changed                     = false;
 
-static GtkWidget * __widget_toolbar_always_show       = 0;
+static GtkWidget * __widget_toolbar_show_behaviour    = 0;
 static GtkWidget * __widget_toolbar_auto_snap         = 0;
 static GtkWidget * __widget_toolbar_hide_timeout      = 0;
 static GtkWidget * __widget_toolbar_show_factory_icon  = 0;
@@ -151,6 +153,18 @@ static GtkWidget * __widget_font                      = 0;
 
 static GtkTooltips * __widget_tooltips                = 0;
 
+enum ToolbarShowFlavourType {
+    SCIM_TOOLBAR_SHOW_ALWAYS,
+    SCIM_TOOLBAR_SHOW_ON_DEMAND,
+    SCIM_TOOLBAR_SHOW_NEVER
+};
+
+static char * __toolbar_show_behaviour_text[] = {
+    N_("Always"),
+    N_("On demand"),
+    N_("Never")
+};
+
 // Declaration of internal functions.
 static void
 on_default_toggle_button_toggled     (GtkToggleButton *togglebutton,
@@ -161,7 +175,7 @@ on_default_spin_button_changed       (GtkSpinButton   *spinbutton,
                                       gpointer         user_data);
 
 static void
-on_toolbar_always_show_toggled       (GtkToggleButton *togglebutton,
+on_toolbar_show_behaviour_changed      (GtkComboBox     *combobox,
                                       gpointer         user_data);
 
 static void
@@ -205,12 +219,29 @@ create_setup_window ()
         gtk_table_set_row_spacings (GTK_TABLE (table), 4);
         gtk_table_set_col_spacings (GTK_TABLE (table), 8);
 
-        __widget_toolbar_always_show = gtk_check_button_new_with_mnemonic (_("Always _show"));
-        gtk_widget_show (__widget_toolbar_always_show);
-        gtk_table_attach (GTK_TABLE (table), __widget_toolbar_always_show, 0, 1, 0, 1,
+        hbox = gtk_hbox_new (FALSE, 0);
+        gtk_widget_show (hbox);
+
+        gtk_table_attach (GTK_TABLE (table), hbox, 0, 1, 0, 1,
                           (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                           (GtkAttachOptions) (GTK_EXPAND), 4, 0);
 
+        label = gtk_label_new_with_mnemonic (_("_Show:"));
+        gtk_widget_show (label);
+        gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 0);
+        gtk_misc_set_padding (GTK_MISC (label), 4, 0);
+ 
+        __widget_toolbar_show_behaviour = gtk_combo_box_new_text ();
+        gtk_combo_box_append_text (GTK_COMBO_BOX (__widget_toolbar_show_behaviour), 
+                                   _(__toolbar_show_behaviour_text[SCIM_TOOLBAR_SHOW_ALWAYS]));
+        gtk_combo_box_append_text (GTK_COMBO_BOX (__widget_toolbar_show_behaviour),
+                                   _(__toolbar_show_behaviour_text[SCIM_TOOLBAR_SHOW_ON_DEMAND]));
+        gtk_combo_box_append_text (GTK_COMBO_BOX (__widget_toolbar_show_behaviour),
+                                   _(__toolbar_show_behaviour_text[SCIM_TOOLBAR_SHOW_NEVER]));
+        gtk_widget_show (__widget_toolbar_show_behaviour);
+        gtk_box_pack_start (GTK_BOX (hbox), __widget_toolbar_show_behaviour, FALSE, FALSE, 0);
+        gtk_label_set_mnemonic_widget (GTK_LABEL (label), __widget_toolbar_show_behaviour);
+
         __widget_toolbar_auto_snap = gtk_check_button_new_with_mnemonic (_("Auto s_nap"));
         gtk_widget_show (__widget_toolbar_auto_snap);
         gtk_table_attach (GTK_TABLE (table), __widget_toolbar_auto_snap, 0, 1, 1, 2,
@@ -327,9 +358,9 @@ create_setup_window ()
         gtk_label_set_mnemonic_widget (GTK_LABEL (label), __widget_font);
 
         // Connect all signals.
-        g_signal_connect ((gpointer) __widget_toolbar_always_show, "toggled",
-                          G_CALLBACK (on_toolbar_always_show_toggled),
-                          &__config_toolbar_always_show);
+        g_signal_connect ((gpointer) __widget_toolbar_show_behaviour, "changed",
+                          G_CALLBACK (on_toolbar_show_behaviour_changed),
+                          NULL);
 
         g_signal_connect ((gpointer) __widget_toolbar_auto_snap, "toggled",
                           G_CALLBACK (on_default_toggle_button_toggled),
@@ -384,11 +415,12 @@ create_setup_window ()
                           NULL);
 
         // Set all tooltips.
-        gtk_tooltips_set_tip (__widget_tooltips, __widget_toolbar_always_show,
-                              _("If this option is checked, "
-                                "the toolbar will be always shown on the screen."
-                                "Otherwise, it will be hidden when SCIM "
-                                "is deactivated."), NULL);
+        gtk_tooltips_set_tip (__widget_tooltips, __widget_toolbar_show_behaviour,
+                              _("If option \"Always\" is selected, "
+                                "the toolbar will always be shown on the screen. "
+                                "If option \"On demand\" is selected, it will only be shown when SCIM "
+                                "is activated. "
+                                "If option \"Never\" is selected, it will never be shown."), NULL);
 
         gtk_tooltips_set_tip (__widget_tooltips, __widget_toolbar_auto_snap,
                               _("If this option is checked, "
@@ -399,7 +431,7 @@ create_setup_window ()
                               _("The toolbar will be hidden out after "
                                 "this timeout is elapsed. "
                                 "This option is only valid when "
-                                "\"Always show\" is checked. "
+                                "\"Always show\" is selected. "
                                 "Set to zero to disable this behavior."), NULL);
 
         gtk_tooltips_set_tip (__widget_tooltips, __widget_toolbar_show_factory_icon,
@@ -460,10 +492,20 @@ create_setup_window ()
 void
 setup_widget_value ()
 {
-    if (__widget_toolbar_always_show) {
-        gtk_toggle_button_set_active (
-            GTK_TOGGLE_BUTTON (__widget_toolbar_always_show),
-            __config_toolbar_always_show);
+    if (__widget_toolbar_show_behaviour) {
+        if (__config_toolbar_always_hidden) {
+            gtk_combo_box_set_active (
+                GTK_COMBO_BOX (__widget_toolbar_show_behaviour),
+                SCIM_TOOLBAR_SHOW_NEVER);
+        } else if (__config_toolbar_always_show) {
+            gtk_combo_box_set_active (
+                GTK_COMBO_BOX (__widget_toolbar_show_behaviour),
+                SCIM_TOOLBAR_SHOW_ALWAYS);
+        } else {
+            gtk_combo_box_set_active (
+                GTK_COMBO_BOX (__widget_toolbar_show_behaviour),
+                SCIM_TOOLBAR_SHOW_ON_DEMAND);
+        }
     }
 
     if (__widget_toolbar_auto_snap) {
@@ -553,6 +595,9 @@ void
 load_config (const ConfigPointer &config)
 {
     if (!config.null ()) {
+        __config_toolbar_always_hidden =
+            config->read (String (SCIM_CONFIG_PANEL_GTK_TOOLBAR_ALWAYS_HIDDEN),
+                          __config_toolbar_always_hidden);
         __config_toolbar_always_show =
             config->read (String (SCIM_CONFIG_PANEL_GTK_TOOLBAR_ALWAYS_SHOW),
                           __config_toolbar_always_show);
@@ -606,6 +651,8 @@ void
 save_config (const ConfigPointer &config)
 {
     if (!config.null ()) {
+        config->write (String (SCIM_CONFIG_PANEL_GTK_TOOLBAR_ALWAYS_HIDDEN),
+                       __config_toolbar_always_hidden);
         config->write (String (SCIM_CONFIG_PANEL_GTK_TOOLBAR_ALWAYS_SHOW),
                        __config_toolbar_always_show);
         config->write (String (SCIM_CONFIG_PANEL_GTK_TOOLBAR_AUTO_SNAP),
@@ -670,17 +717,80 @@ on_default_toggle_button_toggled (GtkToggleButton *togglebutton,
 }
 
 static void
-on_toolbar_always_show_toggled (GtkToggleButton *togglebutton,
-                                gpointer         user_data)
+on_toolbar_show_behaviour_changed (GtkComboBox *combobox,
+                                 gpointer     user_data)
 {
-    __config_toolbar_always_show = gtk_toggle_button_get_active (togglebutton);
+    gint active;
+    active  = gtk_combo_box_get_active (combobox);
+
+    switch (active) {
+        case SCIM_TOOLBAR_SHOW_ALWAYS:
+            __config_toolbar_always_show   = true;
+            __config_toolbar_always_hidden = false;
+            break;
+        case SCIM_TOOLBAR_SHOW_ON_DEMAND:
+            __config_toolbar_always_show   = false;
+            __config_toolbar_always_hidden = false;
+            break;
+        case SCIM_TOOLBAR_SHOW_NEVER:
+            __config_toolbar_always_show   = false;
+            __config_toolbar_always_hidden = true;
+            break;
+        default:
+            __config_toolbar_always_show   = true;
+            __config_toolbar_always_hidden = false;
+            break;
+    }
 
     if (__widget_toolbar_hide_timeout) {
         gtk_widget_set_sensitive (
             __widget_toolbar_hide_timeout,
+            !__config_toolbar_always_hidden &&
             __config_toolbar_always_show);
     }
 
+    if (__widget_toolbar_auto_snap) {
+        gtk_widget_set_sensitive (
+            __widget_toolbar_auto_snap,
+            !__config_toolbar_always_hidden);
+    }
+
+    if (__widget_toolbar_show_factory_icon) {
+        gtk_widget_set_sensitive (
+            __widget_toolbar_show_factory_icon,
+            !__config_toolbar_always_hidden);
+    }
+
+    if (__widget_toolbar_show_factory_name) {
+        gtk_widget_set_sensitive (
+            __widget_toolbar_show_factory_name,
+            !__config_toolbar_always_hidden);
+    }
+
+    if (__widget_toolbar_show_stick_icon) {
+        gtk_widget_set_sensitive (
+            __widget_toolbar_show_stick_icon,
+            !__config_toolbar_always_hidden);
+    }
+
+    if (__widget_toolbar_show_menu_icon) {
+        gtk_widget_set_sensitive (
+            __widget_toolbar_show_menu_icon,
+            !__config_toolbar_always_hidden);
+    }
+
+    if (__widget_toolbar_show_help_icon) {
+        gtk_widget_set_sensitive (
+            __widget_toolbar_show_help_icon,
+            !__config_toolbar_always_hidden);
+    }
+
+    if (__widget_toolbar_show_property_label) {
+        gtk_widget_set_sensitive (
+            __widget_toolbar_show_property_label,
+            !__config_toolbar_always_hidden);
+    }
+
     __have_changed = true;
 }
 
