@@ -23,7 +23,7 @@
  * Free Software Foundation, Inc., 59 Temple Place, Suite 330,
  * Boston, MA  02111-1307  USA
  *
- * $Id: scim_private.cpp,v 1.6 2005/01/10 08:30:54 suzhe Exp $
+ * $Id: scim_private.cpp,v 1.6.4.1 2007/03/11 12:35:10 suzhe Exp $
  *
  */
 
@@ -49,7 +49,9 @@ public:
     TextdomainInitializer () {
 
 #ifdef HAVE_SETLOCALE
-        setlocale (LC_ALL, "");
+        char *locale = setlocale (LC_MESSAGES, 0);
+        if (!locale || *locale == 'C')
+            setlocale (LC_MESSAGES, "");
 #endif
 
         bindtextdomain (GETTEXT_PACKAGE, SCIM_LOCALEDIR);
