@@ -19,7 +19,7 @@
  * Free Software Foundation, Inc., 59 Temple Place, Suite 330,
  * Boston, MA  02111-1307  USA
  *
- * $Id: scim_utility.cpp,v 1.48.2.4 2006/09/24 16:00:52 suzhe Exp $
+ * $Id: scim_utility.cpp,v 1.48.2.5 2006/11/02 04:11:51 suzhe Exp $
  */
 
 #define Uses_SCIM_UTILITY
@@ -32,6 +32,7 @@
 #include <langinfo.h>
 #include <pwd.h>
 #include <dirent.h>
+#include <ctype.h>
 #include <sys/types.h>
 #include <sys/wait.h>
 #include <sys/stat.h>
@@ -816,12 +817,27 @@ __find_language (const String &lang)
     static __Language *langs_begin = __languages;
     static __Language *langs_end   = __languages + sizeof (__languages) / sizeof (__Language) - 1;
 
-    __Language *result = std::lower_bound (langs_begin, langs_end, lang, __LanguageLess ());
+    String nlang = lang;
+    bool contry_code = false;
+
+    // Normalize the language name.
+    for (String::iterator it = nlang.begin (); it != nlang.end (); ++it) {
+        if (*it == '-' || *it == '_') {
+            *it = '_';
+            contry_code = true;
+        } else if (contry_code) {
+            *it = toupper (*it);
+        } else {
+            *it = tolower (*it);
+        }
+    }
+
+    __Language *result = std::lower_bound (langs_begin, langs_end, nlang, __LanguageLess ());
 
     if (result != langs_end) {
-        if (strncmp (result->code, lang.c_str (), strlen (result->code)) == 0 ||
-            (strncmp (result->code, lang.c_str (), lang.length ()) == 0 &&
-             strncmp (result->code, (result+1)->code, lang.length ()) != 0))
+        if (strncmp (result->code, nlang.c_str (), strlen (result->code)) == 0 ||
+            (strncmp (result->code, nlang.c_str (), nlang.length ()) == 0 &&
+             strncmp (result->code, (result+1)->code, nlang.length ()) != 0))
             return result;
     }
 
