@@ -8,6 +8,8 @@
 #include <gtk/gtksettings.h>
 #include <gtk/gtksignal.h>
 #include <gtk/gtkwindow.h>
+#include <gtk/gtkgc.h>
+#include <gtk/gtkstyle.h>
 
 #include "scim_private.h"
 #include "scimstringview.h"
@@ -110,7 +112,7 @@ static void         scim_string_view_keymap_direction_changed (GdkKeymap
 
 static void         scim_string_view_check_cursor_blink       (ScimStringView       *string_view);
 static void         scim_string_view_move_cursor              (ScimStringView       *string_view,
-		                                               guint                 position);
+                                                               guint                 position);
 
 static GtkWidgetClass *parent_class = NULL;
 
@@ -1041,6 +1043,203 @@ scim_string_view_draw_text (ScimStringView *string_view)
     }
 }
 
+#ifndef HAVE_GTK_DRAW_INSERTION_CURSOR
+typedef struct _CursorInfo CursorInfo;
+
+struct _CursorInfo
+{
+  GType for_type;
+  GdkGC *primary_gc;
+  GdkGC *secondary_gc;
+};
+
+static GdkGC *
+make_cursor_gc (GtkWidget      *widget,
+                const gchar    *property_name,
+                const GdkColor *fallback)
+{
+  GdkGCValues gc_values;
+  GdkGCValuesMask gc_values_mask;
+  GdkColor *cursor_color;
+
+  gtk_widget_style_get (widget, property_name, &cursor_color, NULL);
+  
+  gc_values_mask = GDK_GC_FOREGROUND;
+  if (cursor_color)
+    {
+      gc_values.foreground = *cursor_color;
+      gdk_color_free (cursor_color);
+    }
+  else
+    gc_values.foreground = *fallback;
+  
+  gdk_rgb_find_color (widget->style->colormap, &gc_values.foreground);
+  return gtk_gc_get (widget->style->depth, widget->style->colormap, &gc_values, gc_values_mask);
+}
+
+static GdkGC *
+get_insertion_cursor_gc (GtkWidget *widget,
+                         gboolean   is_primary)
+{
+  CursorInfo *cursor_info;
+
+  cursor_info = g_object_get_data (G_OBJECT (widget->style), "gtk-style-cursor-info");
+  if (!cursor_info)
+    {
+      cursor_info = g_new (CursorInfo, 1);
+      g_object_set_data (G_OBJECT (widget->style), "gtk-style-cursor-info", cursor_info);
+      cursor_info->primary_gc = NULL;
+      cursor_info->secondary_gc = NULL;
+      cursor_info->for_type = G_TYPE_INVALID;
+    }
+
+  /* We have to keep track of the type because gtk_widget_style_get()
+   * can return different results when called on the same property and
+   * same style but for different widgets. :-(. That is,
+   * GtkEntry::cursor-color = "red" in a style will modify the cursor
+   * color for entries but not for text view.
+   */
+  if (cursor_info->for_type != G_OBJECT_TYPE (widget))
+    {
+      cursor_info->for_type = G_OBJECT_TYPE (widget);
+      if (cursor_info->primary_gc)
+        {
+          gtk_gc_release (cursor_info->primary_gc);
+          cursor_info->primary_gc = NULL;
+        }
+      if (cursor_info->secondary_gc)
+        {
+          gtk_gc_release (cursor_info->secondary_gc);
+          cursor_info->secondary_gc = NULL;
+        }
+    }
+
+  if (is_primary)
+    {
+      if (!cursor_info->primary_gc)
+        cursor_info->primary_gc = make_cursor_gc (widget,
+                                                  "cursor-color",
+                                                  &widget->style->black);
+        
+      return cursor_info->primary_gc;
+    }
+  else
+    {
+      static const GdkColor gray = { 0, 0x8888, 0x8888, 0x8888 };
+      
+      if (!cursor_info->secondary_gc)
+        cursor_info->secondary_gc = make_cursor_gc (widget,
+                                                    "secondary-cursor-color",
+                                                    &gray);
+        
+      return cursor_info->secondary_gc;
+    }
+}
+
+static void
+draw_insertion_cursor (GtkWidget        *widget,
+                       GdkDrawable      *drawable,
+                       GdkGC            *gc,
+                       GdkRectangle     *location,
+                       GtkTextDirection  direction,
+                       gboolean          draw_arrow)
+{
+  gint stem_width;
+  gint arrow_width;
+  gint x, y;
+  gint i;
+  gfloat cursor_aspect_ratio;
+  gint offset;
+  
+  g_return_if_fail (direction != GTK_TEXT_DIR_NONE);
+  
+  gtk_widget_style_get (widget, "cursor-aspect-ratio", &cursor_aspect_ratio, NULL);
+  
+  stem_width = location->height * cursor_aspect_ratio + 1;
+  arrow_width = stem_width + 1;
+
+  /* put (stem_width % 2) on the proper side of the cursor */
+  if (direction == GTK_TEXT_DIR_LTR)
+    offset = stem_width / 2;
+  else
+    offset = stem_width - stem_width / 2;
+  
+  for (i = 0; i < stem_width; i++)
+    gdk_draw_line (drawable, gc,
+                   location->x + i - offset, location->y,
+                   location->x + i - offset, location->y + location->height - 1);
+
+  if (draw_arrow)
+    {
+      if (direction == GTK_TEXT_DIR_RTL)
+        {
+          x = location->x - offset - 1;
+          y = location->y + location->height - arrow_width * 2 - arrow_width + 1;
+  
+          for (i = 0; i < arrow_width; i++)
+            {
+              gdk_draw_line (drawable, gc,
+                             x, y + i + 1,
+                             x, y + 2 * arrow_width - i - 1);
+              x --;
+            }
+        }
+      else if (direction == GTK_TEXT_DIR_LTR)
+        {
+          x = location->x + stem_width - offset;
+          y = location->y + location->height - arrow_width * 2 - arrow_width + 1;
+  
+          for (i = 0; i < arrow_width; i++) 
+            {
+              gdk_draw_line (drawable, gc,
+                             x, y + i + 1,
+                             x, y + 2 * arrow_width - i - 1);
+              x++;
+            }
+        }
+    }
+}
+
+static void
+scim_string_view_draw_insertion_cursor (GtkWidget        *widget,
+                                        GdkDrawable      *drawable,
+                                        GdkRectangle     *area,
+                                        GdkRectangle     *location,
+                                        gboolean          is_primary,
+                                        GtkTextDirection  direction,
+                                        gboolean          draw_arrow)
+{
+  GdkGC *gc;
+
+  g_return_if_fail (GTK_IS_WIDGET (widget));
+  g_return_if_fail (GDK_IS_DRAWABLE (drawable));
+  g_return_if_fail (location != NULL);
+  g_return_if_fail (direction != GTK_TEXT_DIR_NONE);
+
+  gc = get_insertion_cursor_gc (widget, is_primary);
+  if (area)
+    gdk_gc_set_clip_rectangle (gc, area);
+  
+  draw_insertion_cursor (widget, drawable, gc,
+                         location, direction, draw_arrow);
+  
+  if (area)
+    gdk_gc_set_clip_rectangle (gc, NULL);
+}
+#else
+static void
+scim_string_view_draw_insertion_cursor (GtkWidget        *widget,
+                                        GdkDrawable      *drawable,
+                                        GdkRectangle     *area,
+                                        GdkRectangle     *location,
+                                        gboolean          is_primary,
+                                        GtkTextDirection  direction,
+                                        gboolean          draw_arrow)
+{
+    gtk_draw_insertion_cursor (widget, drawable, area, location, is_primary, direction, draw_arrow);
+}
+#endif
+
 static void
 scim_string_view_draw_cursor (ScimStringView  *string_view)
 {
@@ -1065,10 +1264,6 @@ scim_string_view_draw_cursor (ScimStringView  *string_view)
       gint x1 = 0;
       gint x2 = 0;
 
-#ifndef HAVE_GTK_DRAW_INSERTION_CURSOR
-      GdkGC *gc;
-#endif
-
       gdk_window_get_size (string_view->text_area, &text_area_width, &text_area_height);
 
       scim_string_view_get_cursor_locations (string_view, &strong_x, &weak_x);
@@ -1107,43 +1302,24 @@ scim_string_view_draw_cursor (ScimStringView  *string_view)
       clip_area.width = text_area_width;
       clip_area.height = text_area_height;
 
-#ifndef HAVE_GTK_DRAW_INSERTION_CURSOR
-      gc = _gtk_get_insertion_cursor_gc (widget, TRUE);
-      _gtk_draw_insertion_cursor (widget,
-                                  string_view->text_area,
-                                  gc,
-                                  &cursor_location,
-                                  dir1,
-                                  dir2 != GTK_TEXT_DIR_NONE );
-      g_object_unref (gc);
-#else
-      gtk_draw_insertion_cursor (widget,
-                                 string_view->text_area,
-                                 &clip_area,
-                                 &cursor_location,
-                                 TRUE,
-                                 dir1,
-                                 dir2 != GTK_TEXT_DIR_NONE);
-#endif
-      
+      scim_string_view_draw_insertion_cursor (widget,
+                                              string_view->text_area,
+                                              &clip_area,
+                                              &cursor_location,
+                                              TRUE,
+                                              dir1,
+                                              dir2 != GTK_TEXT_DIR_NONE);
+
       if (dir2 != GTK_TEXT_DIR_NONE)
         {
           cursor_location.x = xoffset + x2;
-#ifndef HAVE_GTK_DRAW_INSERTION_CURSOR
-          gc = _gtk_get_insertion_cursor_gc (widget, FALSE);
-          _gtk_draw_insertion_cursor (widget, string_view->text_area, gc,
-                                      &cursor_location, dir2,
-                                      TRUE);
-          g_object_unref (gc);
-#else
-          gtk_draw_insertion_cursor (widget,
-                                     string_view->text_area,
-                                     &clip_area,
-                                     &cursor_location,
-                                     FALSE,
-                                     dir1,
-                                     TRUE);
-#endif
+          scim_string_view_draw_insertion_cursor (widget,
+                                                  string_view->text_area,
+                                                  &clip_area,
+                                                  &cursor_location,
+                                                  FALSE,
+                                                  dir2,
+                                                  TRUE);
         }
     }
 }
